@@ -19,6 +19,9 @@ configs = {
                                         # Laptop Windows -1
     'buffersize'      : -1,             # default is 4 for V4L2, max 10, 
                                         # Laptop: -1
+    'gain'            : 4,              # Sensor readout gain
+    'autowb'          : 1,              # enable auto white balancing
+    'wb_temp'         : 4600,           # white balance temperature
     ##############################################
     # Target Recognition
     ##############################################
